from dataclasses import dataclass, field
from typing import Optional
import logging

import configparser
import os


from dotenv import load_dotenv


def default_logging(main_logging: logging) -> None:
    log_dir: str = os.path.join(
        os.path.dirname(os.path.abspath(os.path.dirname(__file__))), "logs"
    )
    main_logging.basicConfig(
        level=20,
        datefmt="%Y-%m-%d %H:%M:%S",
        format="[%(asctime)s.%(msecs)03d] %(module)-25s:%(lineno)4d | %(funcName)-20s| %(levelname)-8s | %(message)s",
    )
    file_handler = main_logging.FileHandler(os.path.join(log_dir, "logfile.log"))
    file_handler.setLevel(30)
    file_handler.setFormatter(
        logging.Formatter(
            "[%(asctime)s.%(msecs)03d] %(module)s:%(lineno)4d | %(funcName)s| %(levelname)s | %(message)s"
        )
    )
    main_logging.getLogger().addHandler(file_handler)
    main_logging.info("success configure logging")


default_logging(logging)


def configure_loging(main_logging: logging) -> None:
    log_dir: str = os.path.join(
        os.path.dirname(os.path.abspath(os.path.dirname(__file__))), "logs"
    )
    logging.info("configure logging")
    main_logging.basicConfig(
        level=cfg.log.console_lvl,
        datefmt=cfg.log.dare_format,
        format=cfg.log.format,
    )
    handler = main_logging.FileHandler(os.path.join(log_dir, "logfile.log"))
    handler.setLevel(cfg.log.file_lvl)
    handler.setFormatter(logging.Formatter(cfg.log.format))
    logging.getLogger().addHandler(handler)
    logging.info("success configure logging")


def load_config():
    cfg_dir: str = os.path.abspath(os.path.dirname(__file__))
    logging.info("loading env")
    load_dotenv(dotenv_path=os.path.join(cfg_dir, ".env"))
    logging.info("success loading env")

    logging.info("loading config")
    loaded_config: configparser.ConfigParser = configparser.ConfigParser(
        interpolation=None
    )
    user_config: str = os.path.join(cfg_dir, "config.ini")
    loaded_config.read(user_config)
    logging.info("success loading config")
    return loaded_config


config = load_config()


@dataclass
class ConfigLog:
    console_lvl: int = config["log"].getint("console_lvl")
    file_lvl: int = config["log"].getint("file_lvl")
    dare_format: str = config["log"].get("dare_format")
    format: str = config["log"].get("format")


@dataclass
class ConfigTelegram:
    send: bool = config["tg"].getboolean("send")
    if send:
        bot_token: str = os.getenv("TG_BOT_TOKEN")
        chat_id: int = int(os.getenv("TG_CHAT_ID"))


@dataclass()
class ConfigAll:
    log: ConfigLog = ConfigLog
    tg: ConfigTelegram = ConfigTelegram


cfg = ConfigAll

configure_loging(logging)
